var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
import { DummyClassifier } from '../dummy/dummyClassifier.js';
import { DummyRegressor } from '../dummy/dummyRegressor.js';
import { LogisticRegression } from '../linear_model/logisticRegression.js';
import { RidgeRegression } from '../linear_model/ridgeRegression.js';
import { LinearRegression } from '../linear_model/linearRegression.js';
import { LassoRegression } from '../linear_model/lassoRegression.js';
import { ElasticNet } from '../linear_model/elasticNet.js';
import { LabelEncoder } from '../preprocessing/labelEncoder.js';
import { SimpleImputer } from '../impute/simpleImputer.js';
import { tf } from '../shared-esm/globals.js';
import { MinMaxScaler } from '../preprocessing/minMaxScaler.js';
function getEstimator(name, serialJson) {
    switch (name) {
        case 'dummyclassifier':
            return new DummyClassifier().fromJson(serialJson);
        case 'dummyregressor':
            return new DummyRegressor().fromJson(serialJson);
        case 'logisticregression':
            return new LogisticRegression().fromJson(serialJson);
        case 'ridgeregression':
            return new RidgeRegression().fromJson(serialJson);
        case 'linearregression':
            return new LinearRegression().fromJson(serialJson);
        case 'lassoregression':
            return new LassoRegression().fromJson(serialJson);
        case 'elasticnet':
            return new ElasticNet().fromJson(serialJson);
        case 'simpleimputer':
            return new SimpleImputer().fromJson(serialJson);
        case 'minmaxscaler':
            return new MinMaxScaler().fromJson(serialJson);
        default:
            throw new Error("".concat(name, " estimator not supported"));
    }
}
export function fromJson(classConstructor, model) {
    var e_1, _a;
    var jsonClass = JSON.parse(model);
    if (jsonClass.name != classConstructor.name) {
        throw new Error("wrong json values for ".concat(classConstructor.name, " constructor"));
    }
    var copyThis = Object.assign({}, classConstructor);
    try {
        for (var _b = __values(Object.keys(classConstructor)), _c = _b.next(); !_c.done; _c = _b.next()) {
            var key = _c.value;
            var value = copyThis[key];
            if (value instanceof tf.Tensor) {
                jsonClass[key] = tf.tensor(jsonClass[key]);
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
    // for ensembles
    if (jsonClass.estimators || jsonClass.steps) {
        var jsonEstimatorOrStep = jsonClass.estimators || jsonClass.steps;
        for (var i = 0; i < jsonEstimatorOrStep.length; i++) {
            var estimatorName = JSON.parse(jsonEstimatorOrStep[i][1]).name;
            var estimators = getEstimator(estimatorName, jsonEstimatorOrStep[i][1]);
            jsonEstimatorOrStep[i][1] = Object.assign(estimators, jsonEstimatorOrStep[i][1]);
        }
    }
    if (jsonClass.le) {
        var labelEncode = new LabelEncoder();
        jsonClass.le = Object.assign(labelEncode, jsonClass.le);
    }
    return Object.assign(classConstructor, jsonClass);
}
export function toJson(classConstructor, classJson) {
    return __awaiter(this, void 0, void 0, function () {
        var i, _a, _b, estimator, _c, _d, e_2_1, _e, _f, step, _g, _h, e_3_1;
        var e_2, _j, e_3, _k;
        return __generator(this, function (_l) {
            switch (_l.label) {
                case 0:
                    i = 0;
                    if (!classConstructor.estimators) return [3 /*break*/, 8];
                    _l.label = 1;
                case 1:
                    _l.trys.push([1, 6, 7, 8]);
                    _a = __values(classConstructor.estimators), _b = _a.next();
                    _l.label = 2;
                case 2:
                    if (!!_b.done) return [3 /*break*/, 5];
                    estimator = _b.value;
                    _c = classJson.estimators[i];
                    _d = 1;
                    return [4 /*yield*/, estimator[1].toJson()];
                case 3:
                    _c[_d] = _l.sent();
                    i += 1;
                    _l.label = 4;
                case 4:
                    _b = _a.next();
                    return [3 /*break*/, 2];
                case 5: return [3 /*break*/, 8];
                case 6:
                    e_2_1 = _l.sent();
                    e_2 = { error: e_2_1 };
                    return [3 /*break*/, 8];
                case 7:
                    try {
                        if (_b && !_b.done && (_j = _a.return)) _j.call(_a);
                    }
                    finally { if (e_2) throw e_2.error; }
                    return [7 /*endfinally*/];
                case 8:
                    if (!classConstructor.steps) return [3 /*break*/, 16];
                    _l.label = 9;
                case 9:
                    _l.trys.push([9, 14, 15, 16]);
                    _e = __values(classConstructor.steps), _f = _e.next();
                    _l.label = 10;
                case 10:
                    if (!!_f.done) return [3 /*break*/, 13];
                    step = _f.value;
                    _g = classJson.steps[i];
                    _h = 1;
                    return [4 /*yield*/, step[1].toJson()];
                case 11:
                    _g[_h] = _l.sent();
                    i += 1;
                    _l.label = 12;
                case 12:
                    _f = _e.next();
                    return [3 /*break*/, 10];
                case 13: return [3 /*break*/, 16];
                case 14:
                    e_3_1 = _l.sent();
                    e_3 = { error: e_3_1 };
                    return [3 /*break*/, 16];
                case 15:
                    try {
                        if (_f && !_f.done && (_k = _e.return)) _k.call(_e);
                    }
                    finally { if (e_3) throw e_3.error; }
                    return [7 /*endfinally*/];
                case 16: return [2 /*return*/, JSON.stringify(classJson)];
            }
        });
    });
}
