import { tf } from '../shared-esm/globals.js';
import { Scikit1D, Scikit2D } from '../types.js';
import Serialize from '../serialize.js';
export interface NaiveBayesParams {
    /**
     * Prior probabilities of the classes. If specified the priors are not adjusted according to the data.
     */
    priors?: Scikit1D;
    /**
     * Portion of the largest variance of all features that is added to variances for calculation stability.
     * Default value is 1e-9
     */
    varSmoothing?: number;
}
export declare abstract class BaseNaiveBayes extends Serialize {
    priors?: tf.Tensor1D;
    varSmoothing: number;
    classes: tf.Tensor1D;
    means: tf.Tensor1D[];
    variances: tf.Tensor1D[];
    constructor(params?: NaiveBayesParams);
    /**
     * Train the model by calculating the mean and variance of sample distribution.
     * @param X
     * @param y
     * @returns
     */
    fit(X: Scikit2D, y: Scikit1D): Promise<this>;
    /**
     * Predict the probability of samples assigned to each observed label.
     * @param X
     * @returns {tf.Tensor} Probabilities
     */
    predictProba(X: Scikit2D): tf.Tensor<tf.Rank>;
    /**
     * Predict the labels assigned to each sample
     * @param X
     * @returns {tf.Tensor} Labels
     */
    predict(X: Scikit2D): tf.Tensor<tf.Rank>;
    /**
     * Kernel function to calculate posterior probability, which should be implemented in the subclass.
     * @param features
     * @param mean
     * @param variance
     */
    protected abstract kernel(features: tf.Tensor2D, mean: tf.Tensor1D, variance: tf.Tensor1D): tf.Tensor1D;
    toJson(): string;
    fromJson(model: string): this;
}
