var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { assert } from '../typesUtils.js';
import Serialize from '../serialize.js';
export function giniCoefficient(labelFreqs, nSamples) {
    var freqSquares = 0;
    for (var i = 0; i < labelFreqs.length; i++) {
        freqSquares += labelFreqs[i] * labelFreqs[i];
    }
    return 1 - freqSquares / (nSamples * nSamples);
}
export function entropy(labelFreqs, nSamples) {
    var totalEntropy = 0;
    for (var i = 0; i < labelFreqs.length; i++) {
        var labelFrequency = labelFreqs[i];
        if (labelFrequency > 0) {
            labelFrequency /= nSamples;
            totalEntropy -= labelFrequency * Math.log2(labelFrequency);
        }
    }
    return totalEntropy;
}
export function mse(ySquaredSum, ySum, nSamples) {
    var yBar = ySum / nSamples;
    var val = ySquaredSum / nSamples - yBar * yBar;
    return val;
}
function arrayMax(labels) {
    var max = Number.NEGATIVE_INFINITY;
    for (var i = 0; i < labels.length; i++) {
        if (labels[i] > max) {
            max = labels[i];
        }
    }
    return max;
}
var ClassificationCriterion = /** @class */ (function (_super) {
    __extends(ClassificationCriterion, _super);
    function ClassificationCriterion(impurityMeasure, y) {
        var _this = _super.call(this) || this;
        _this.start = 0;
        _this.end = 0;
        _this.pos = 0;
        _this.labelFreqsTotal = [];
        _this.labelFreqsLeft = [];
        _this.labelFreqsRight = [];
        _this.nSamples = 0;
        _this.nSamplesLeft = 0;
        _this.nSamplesRight = 0;
        _this.name = 'classificationCriterion';
        assert(['gini', 'entropy'].includes(impurityMeasure), 'Unkown impurity measure. Only supports gini, and entropy');
        _this.impurityMeasure = impurityMeasure;
        if (_this.impurityMeasure === 'gini') {
            _this.impurityFunc = giniCoefficient;
        }
        else {
            _this.impurityFunc = entropy;
        }
        // This assumes that the labels are 0,1,2,...,(n-1)
        _this.nLabels = arrayMax(y) + 1;
        _this.y = y;
        _this.labelFreqsTotal = new Array(_this.nLabels).fill(0);
        _this.labelFreqsLeft = new Array(_this.nLabels).fill(0);
        _this.labelFreqsRight = new Array(_this.nLabels).fill(0);
        return _this;
    }
    ClassificationCriterion.prototype.init = function (start, end, sampleMap) {
        this.start = start;
        this.end = end;
        this.nSamples = end - start;
        this.labelFreqsTotal = this.labelFreqsTotal.fill(0);
        this.labelFreqsLeft = this.labelFreqsLeft.fill(0);
        this.labelFreqsRight = this.labelFreqsRight.fill(0);
        for (var i = start; i < end; i++) {
            var sampleNumber = sampleMap[i];
            this.labelFreqsTotal[this.y[sampleNumber]] += 1;
        }
    };
    ClassificationCriterion.prototype.reset = function () {
        this.pos = this.start;
        this.labelFreqsLeft = this.labelFreqsLeft.fill(0);
        this.labelFreqsRight = this.labelFreqsRight.fill(0);
    };
    ClassificationCriterion.prototype.update = function (newPos, sampleMap) {
        for (var i = this.pos; i < newPos; i++) {
            // This assumes that the labels take values 0,..., nLabels - 1
            var sampleNumber = sampleMap[i];
            this.labelFreqsLeft[this.y[sampleNumber]] += 1;
        }
        // calculate labelFreqsRight
        for (var i = 0; i < this.labelFreqsTotal.length; i++) {
            this.labelFreqsRight[i] =
                this.labelFreqsTotal[i] - this.labelFreqsLeft[i];
        }
        this.pos = newPos;
        this.nSamplesLeft = this.pos - this.start;
        this.nSamplesRight = this.end - this.pos;
    };
    ClassificationCriterion.prototype.childrenImpurities = function () {
        return {
            impurityLeft: this.impurityFunc(this.labelFreqsLeft, this.nSamplesLeft),
            impurityRight: this.impurityFunc(this.labelFreqsRight, this.nSamplesRight)
        };
    };
    ClassificationCriterion.prototype.impurityImprovement = function () {
        var _a = this.childrenImpurities(), impurityLeft = _a.impurityLeft, impurityRight = _a.impurityRight;
        return (-this.nSamplesLeft * impurityLeft - this.nSamplesRight * impurityRight);
    };
    ClassificationCriterion.prototype.nodeImpurity = function () {
        return this.impurityFunc(this.labelFreqsTotal, this.nSamples);
    };
    ClassificationCriterion.prototype.nodeValue = function () {
        return this.labelFreqsTotal;
    };
    ClassificationCriterion.fromJson = function (model) {
        var jsonClass = JSON.parse(model);
        var newModel = new ClassificationCriterion(jsonClass.impurityMeasure, jsonClass.y);
        return Object.assign(newModel, jsonClass);
    };
    return ClassificationCriterion;
}(Serialize));
export { ClassificationCriterion };
var RegressionCriterion = /** @class */ (function (_super) {
    __extends(RegressionCriterion, _super);
    function RegressionCriterion(impurityMeasure, y) {
        var _this = _super.call(this) || this;
        _this.start = 0;
        _this.end = 0;
        _this.pos = 0;
        _this.squaredSum = 0;
        _this.squaredSumLeft = 0;
        _this.squaredSumRight = 0;
        _this.sumTotal = 0;
        _this.sumTotalLeft = 0;
        _this.sumTotalRight = 0;
        _this.nSamples = 0;
        _this.nSamplesLeft = 0;
        _this.nSamplesRight = 0;
        _this.name = 'regressionCriterion';
        assert(['squared_error'].includes(impurityMeasure), 'Unkown impurity measure. Only supports squared_error');
        // Support MAE one day
        _this.impurityMeasure = impurityMeasure;
        _this.impurityFunc = mse;
        _this.y = y;
        return _this;
    }
    RegressionCriterion.prototype.init = function (start, end, sampleMap) {
        this.sumTotal = 0;
        this.squaredSum = 0;
        this.start = start;
        this.end = end;
        this.nSamples = end - start;
        for (var i = start; i < end; i++) {
            var sampleNumber = sampleMap[i];
            var yValue = this.y[sampleNumber];
            this.sumTotal += yValue;
            this.squaredSum += yValue * yValue;
        }
    };
    RegressionCriterion.prototype.reset = function () {
        this.pos = this.start;
        this.squaredSumLeft = 0;
        this.sumTotalLeft = 0;
        this.squaredSumRight = 0;
        this.sumTotalRight = 0;
    };
    RegressionCriterion.prototype.update = function (newPos, sampleMap) {
        for (var i = this.pos; i < newPos; i++) {
            // This assumes that the labels take values 0,..., nLabels - 1
            var sampleNumber = sampleMap[i];
            var yValue = this.y[sampleNumber];
            this.sumTotalLeft += yValue;
            this.squaredSumLeft += yValue * yValue;
        }
        // calculate labelFreqsRight
        this.sumTotalRight = this.sumTotal - this.sumTotalLeft;
        this.squaredSumRight = this.squaredSum - this.squaredSumLeft;
        this.pos = newPos;
        this.nSamplesLeft = this.pos - this.start;
        this.nSamplesRight = this.end - this.pos;
    };
    RegressionCriterion.prototype.childrenImpurities = function () {
        return {
            impurityLeft: this.impurityFunc(this.squaredSumLeft, this.sumTotalLeft, this.nSamplesLeft),
            impurityRight: this.impurityFunc(this.squaredSumRight, this.sumTotalRight, this.nSamplesRight)
        };
    };
    RegressionCriterion.prototype.impurityImprovement = function () {
        var _a = this.childrenImpurities(), impurityLeft = _a.impurityLeft, impurityRight = _a.impurityRight;
        return (-this.nSamplesLeft * impurityLeft - this.nSamplesRight * impurityRight);
    };
    RegressionCriterion.prototype.nodeImpurity = function () {
        return this.impurityFunc(this.squaredSum, this.sumTotal, this.nSamples);
    };
    RegressionCriterion.prototype.nodeValue = function () {
        return [this.sumTotal / this.nSamples];
    };
    RegressionCriterion.fromJson = function (model) {
        var jsonClass = JSON.parse(model);
        var newModel = new RegressionCriterion(jsonClass.impurityMeasure, jsonClass.y);
        return Object.assign(newModel, jsonClass);
    };
    return RegressionCriterion;
}(Serialize));
export { RegressionCriterion };
