import { DummyClassifier } from '../dummy/dummyClassifier.js';
import { DummyRegressor } from '../dummy/dummyRegressor.js';
import { LogisticRegression } from '../linear_model/logisticRegression.js';
import { RidgeRegression } from '../linear_model/ridgeRegression.js';
import { LinearRegression } from '../linear_model/linearRegression.js';
import { LassoRegression } from '../linear_model/lassoRegression.js';
import { ElasticNet } from '../linear_model/elasticNet.js';
import { LabelEncoder } from '../preprocessing/labelEncoder.js';
import { SimpleImputer } from '../impute/simpleImputer.js';
import { tf } from '../shared-esm/globals.js';
import { MinMaxScaler } from '../preprocessing/minMaxScaler.js';
function getEstimator(name, serialJson) {
    switch (name) {
        case 'dummyclassifier':
            return new DummyClassifier().fromJson(serialJson);
        case 'dummyregressor':
            return new DummyRegressor().fromJson(serialJson);
        case 'logisticregression':
            return new LogisticRegression().fromJson(serialJson);
        case 'ridgeregression':
            return new RidgeRegression().fromJson(serialJson);
        case 'linearregression':
            return new LinearRegression().fromJson(serialJson);
        case 'lassoregression':
            return new LassoRegression().fromJson(serialJson);
        case 'elasticnet':
            return new ElasticNet().fromJson(serialJson);
        case 'simpleimputer':
            return new SimpleImputer().fromJson(serialJson);
        case 'minmaxscaler':
            return new MinMaxScaler().fromJson(serialJson);
        default:
            throw new Error(`${name} estimator not supported`);
    }
}
export function fromJson(classConstructor, model) {
    let jsonClass = JSON.parse(model);
    if (jsonClass.name != classConstructor.name) {
        throw new Error(`wrong json values for ${classConstructor.name} constructor`);
    }
    const copyThis = Object.assign({}, classConstructor);
    for (let key of Object.keys(classConstructor)) {
        let value = copyThis[key];
        if (value instanceof tf.Tensor) {
            jsonClass[key] = tf.tensor(jsonClass[key]);
        }
    }
    // for ensembles
    if (jsonClass.estimators || jsonClass.steps) {
        const jsonEstimatorOrStep = jsonClass.estimators || jsonClass.steps;
        for (let i = 0; i < jsonEstimatorOrStep.length; i++) {
            const estimatorName = JSON.parse(jsonEstimatorOrStep[i][1]).name;
            const estimators = getEstimator(estimatorName, jsonEstimatorOrStep[i][1]);
            jsonEstimatorOrStep[i][1] = Object.assign(estimators, jsonEstimatorOrStep[i][1]);
        }
    }
    if (jsonClass.le) {
        const labelEncode = new LabelEncoder();
        jsonClass.le = Object.assign(labelEncode, jsonClass.le);
    }
    return Object.assign(classConstructor, jsonClass);
}
export async function toJson(classConstructor, classJson) {
    let i = 0;
    if (classConstructor.estimators) {
        for (const estimator of classConstructor.estimators) {
            classJson.estimators[i][1] = await estimator[1].toJson();
            i += 1;
        }
    }
    if (classConstructor.steps) {
        for (const step of classConstructor.steps) {
            classJson.steps[i][1] = await step[1].toJson();
            i += 1;
        }
    }
    return JSON.stringify(classJson);
}
