/**
*  @license
* Copyright 2021, JsData. All rights reserved.
*
* This source code is licensed under the MIT license found in the
* LICENSE file in the root directory of this source tree.

* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
* ==========================================================================
*/
import { tf } from '../shared-esm/globals.js';
import { Neighborhood, NeighborhoodParams } from './neighborhood.js';
/**
 * A {@link Neighborhood} implementation using a kd-tree as data structure.
 * Instead of an object-oriented representation, the implementation uses an
 * inorder array-based representation of the tree, similar to binary heaps.
 * The tree is always balanced. It is constructed by recursively spliting
 * up the largest dimension of the axis-aligned bounding box of the remaining
 * set of points.
 */
export declare class KdTree implements Neighborhood {
    private _nSamples;
    private _nFeatures;
    private _metric;
    /**
     * Coordinates of the points contained in this kdTree, not in the order
     * as they were passed to {@link KdTree.build}.
     */
    private _points;
    /**
     * Keeps track of the order, in which the points were originally passed
     * to {@link KdTree.build}. The `i+1`-th point in `_points` was originally
     * passed as `_indices[i]+1`-th point to {@link KdTree.build}.
     */
    private _indices;
    /**
     * The bounding box of each tree node.
     */
    private _bBoxes;
    /**
     * The (i+1)-th leaf of this tree contains the points
     * `_points[_offsets[i]]` to `_points[_offsets[i+1]-1]`.
     */
    private _offsets;
    private constructor();
    /**
     * Asynchronously builds a {@link KdTree}.
     */
    static build({ metric, entries, leafSize }: NeighborhoodParams): Promise<KdTree>;
    kNearest(k: number, queryPoints: tf.Tensor2D): {
        distances: tf.Tensor2D;
        indices: tf.Tensor2D;
    };
}
