"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toJson = exports.fromJson = void 0;
const dummyClassifier_1 = require("../dummy/dummyClassifier.js");
const dummyRegressor_1 = require("../dummy/dummyRegressor.js");
const logisticRegression_1 = require("../linear_model/logisticRegression.js");
const ridgeRegression_1 = require("../linear_model/ridgeRegression.js");
const linearRegression_1 = require("../linear_model/linearRegression.js");
const lassoRegression_1 = require("../linear_model/lassoRegression.js");
const elasticNet_1 = require("../linear_model/elasticNet.js");
const labelEncoder_1 = require("../preprocessing/labelEncoder.js");
const simpleImputer_1 = require("../impute/simpleImputer.js");
const globals_1 = require("../shared-node/globals.js");
const minMaxScaler_1 = require("../preprocessing/minMaxScaler.js");
function getEstimator(name, serialJson) {
    switch (name) {
        case 'dummyclassifier':
            return new dummyClassifier_1.DummyClassifier().fromJson(serialJson);
        case 'dummyregressor':
            return new dummyRegressor_1.DummyRegressor().fromJson(serialJson);
        case 'logisticregression':
            return new logisticRegression_1.LogisticRegression().fromJson(serialJson);
        case 'ridgeregression':
            return new ridgeRegression_1.RidgeRegression().fromJson(serialJson);
        case 'linearregression':
            return new linearRegression_1.LinearRegression().fromJson(serialJson);
        case 'lassoregression':
            return new lassoRegression_1.LassoRegression().fromJson(serialJson);
        case 'elasticnet':
            return new elasticNet_1.ElasticNet().fromJson(serialJson);
        case 'simpleimputer':
            return new simpleImputer_1.SimpleImputer().fromJson(serialJson);
        case 'minmaxscaler':
            return new minMaxScaler_1.MinMaxScaler().fromJson(serialJson);
        default:
            throw new Error(`${name} estimator not supported`);
    }
}
function fromJson(classConstructor, model) {
    let jsonClass = JSON.parse(model);
    if (jsonClass.name != classConstructor.name) {
        throw new Error(`wrong json values for ${classConstructor.name} constructor`);
    }
    const copyThis = Object.assign({}, classConstructor);
    for (let key of Object.keys(classConstructor)) {
        let value = copyThis[key];
        if (value instanceof globals_1.tf.Tensor) {
            jsonClass[key] = globals_1.tf.tensor(jsonClass[key]);
        }
    }
    // for ensembles
    if (jsonClass.estimators || jsonClass.steps) {
        const jsonEstimatorOrStep = jsonClass.estimators || jsonClass.steps;
        for (let i = 0; i < jsonEstimatorOrStep.length; i++) {
            const estimatorName = JSON.parse(jsonEstimatorOrStep[i][1]).name;
            const estimators = getEstimator(estimatorName, jsonEstimatorOrStep[i][1]);
            jsonEstimatorOrStep[i][1] = Object.assign(estimators, jsonEstimatorOrStep[i][1]);
        }
    }
    if (jsonClass.le) {
        const labelEncode = new labelEncoder_1.LabelEncoder();
        jsonClass.le = Object.assign(labelEncode, jsonClass.le);
    }
    return Object.assign(classConstructor, jsonClass);
}
exports.fromJson = fromJson;
async function toJson(classConstructor, classJson) {
    let i = 0;
    if (classConstructor.estimators) {
        for (const estimator of classConstructor.estimators) {
            classJson.estimators[i][1] = await estimator[1].toJson();
            i += 1;
        }
    }
    if (classConstructor.steps) {
        for (const step of classConstructor.steps) {
            classJson.steps[i][1] = await step[1].toJson();
            i += 1;
        }
    }
    return JSON.stringify(classJson);
}
exports.toJson = toJson;
