"use strict";
/**
*  @license
* Copyright 2021, JsData. All rights reserved.
*
* This source code is licensed under the MIT license found in the
* LICENSE file in the root directory of this source tree.

* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
* ==========================================================================
*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.ElasticNet = void 0;
const sgdRegressor_1 = require("./sgdRegressor.js");
const globals_1 = require("../shared-node/globals.js");
/**
 * Linear regression with combined L1 and L2 priors as regularizer.
 */
class ElasticNet extends sgdRegressor_1.SGDRegressor {
    /** Useful for pipelines and column transformers to have a default name for transforms */
    name = 'elasticnet';
    constructor({ alpha = 1, l1Ratio = 0.5, fitIntercept = true } = {}) {
        super({
            modelCompileArgs: {
                optimizer: globals_1.tf.train.adam(0.1),
                loss: globals_1.tf.losses.meanSquaredError,
                metrics: ['mse']
            },
            modelFitArgs: {
                batchSize: 32,
                epochs: 1000,
                verbose: 0,
                callbacks: [
                    globals_1.tf.callbacks.earlyStopping({ monitor: 'mse', patience: 50 })
                ]
            },
            denseLayerArgs: {
                units: 1,
                kernelRegularizer: globals_1.tf.regularizers.l1l2({
                    l1: alpha * l1Ratio,
                    l2: 0.5 * alpha * (1 - l1Ratio)
                }),
                useBias: Boolean(fitIntercept)
            },
            optimizerType: 'adam',
            lossType: 'meanSquaredError'
        });
    }
}
exports.ElasticNet = ElasticNet;
