"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fromJson = exports.toJSON = void 0;
const utils_1 = require("../utils.js");
const globals_1 = require("../shared-node/globals.js");
const oneHotEncoder_1 = require("../preprocessing/oneHotEncoder.js");
function getModelWeight(model) {
    return Promise.all(model.getWeights().map((weight) => weight.array()));
}
async function toJSON(classConstructor, classifierJson) {
    const modelConfig = classConstructor.model.getConfig();
    const modelWeight = await getModelWeight(classConstructor.model);
    classifierJson.model = {
        config: modelConfig,
        weight: modelWeight
    };
    if (classConstructor.denseLayerArgs.kernelInitializer) {
        const initializerName = classConstructor.denseLayerArgs.kernelInitializer.constructor.name;
        classifierJson.denseLayerArgs.kernelInitializer = initializerName;
    }
    if (classConstructor.denseLayerArgs.biasInitializer) {
        const biasName = classConstructor.denseLayerArgs.biasInitializer.constructor.name;
        classifierJson.denseLayerArgs.biasInitializer = biasName;
    }
    // set optimizer
    classifierJson.modelCompileArgs.optimizer =
        classConstructor.model.optimizer.getConfig();
    return JSON.stringify(classifierJson);
}
exports.toJSON = toJSON;
function fromJson(classConstructor, model) {
    let jsonClass = JSON.parse(model);
    if (jsonClass.name != classConstructor.name) {
        throw new Error(`wrong json values for ${classConstructor.name} constructor`);
    }
    const jsonModel = globals_1.tf.Sequential.fromConfig(globals_1.tf.Sequential, jsonClass.model.config);
    const jsonOpt = (0, utils_1.optimizer)(jsonClass.optimizerType);
    const optim = Object.assign(jsonOpt, jsonClass.modelCompileArgs.optimizer);
    const loss = (0, utils_1.getLoss)(jsonClass.lossType);
    jsonClass.modelCompileArgs = {
        ...jsonClass.modelCompileArgs,
        optimizer: optim,
        loss: loss
    };
    jsonModel.compile(jsonClass.modelCompileArgs);
    const weights = [];
    for (const weight of jsonClass.model.weight) {
        weights.push(globals_1.tf.tensor(weight));
    }
    jsonModel.setWeights(weights);
    jsonClass.model = jsonModel;
    // if call back create callback
    // default usecase is set to EarlyStop
    // might get complex for custom callback
    if (jsonClass.modelFitArgs.callbacks) {
        let jsonCallback = globals_1.tf.callbacks.earlyStopping();
        let modelFitArgs = jsonClass.modelFitArgs;
        jsonCallback = Object.assign(jsonCallback, modelFitArgs.callbacks[0]);
        modelFitArgs.callbacks = [jsonCallback];
    }
    if (jsonClass.denseLayerArgs.kernelInitializer) {
        let initializerName = jsonClass.denseLayerArgs.kernelInitializer;
        jsonClass.denseLayerArgs.kernelInitializer = (0, utils_1.initializer)(initializerName);
    }
    if (jsonClass.denseLayerArgs.biasInitializer) {
        let biasName = jsonClass.denseLayerArgs.biasInitializer;
        jsonClass.denseLayerArgs.biasInitializer = (0, utils_1.initializer)(biasName);
    }
    if (jsonClass.oneHot) {
        let jsonOneHotEncoder = new oneHotEncoder_1.OneHotEncoder();
        jsonClass.oneHot = Object.assign(jsonOneHotEncoder, jsonClass.oneHot);
    }
    return Object.assign(classConstructor, jsonClass);
}
exports.fromJson = fromJson;
