"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mixins = exports.ClassifierMixin = exports.RegressorMixin = exports.PredictorMixin = exports.TransformerMixin = void 0;
const metrics_1 = require("./metrics/metrics.js");
const serialize_1 = __importDefault(require("./serialize.js"));
class TransformerMixin extends serialize_1.default {
    fitTransform(X) {
        return this.fit(X).transform(X);
    }
}
exports.TransformerMixin = TransformerMixin;
class PredictorMixin {
    fitPredict(X, y) {
        return this.fit(X, y).predict(X);
    }
}
exports.PredictorMixin = PredictorMixin;
class RegressorMixin extends serialize_1.default {
    EstimatorType = 'regressor';
    score(X, y) {
        const yPred = this.predict(X);
        return (0, metrics_1.r2Score)(y, yPred);
    }
}
exports.RegressorMixin = RegressorMixin;
class ClassifierMixin extends serialize_1.default {
    EstimatorType = 'classifier';
    score(X, y) {
        const yPred = this.predict(X);
        return (0, metrics_1.accuracyScore)(y, yPred);
    }
}
exports.ClassifierMixin = ClassifierMixin;
const mixins = (baseClass, ...mixins) => {
    class base extends baseClass {
        constructor(...args) {
            super(...args);
            mixins.forEach((mixin) => {
                copyProps(this, new mixin());
            });
        }
    }
    let copyProps = (target, source) => {
        // this function copies all properties and symbols, filtering out some special ones
        Object.getOwnPropertyNames(source)
            .concat(Object.getOwnPropertySymbols(source))
            .forEach((prop) => {
            if (!prop.match(/^(?:constructor|prototype|arguments|caller|name|bind|call|apply|toString|length)$/))
                Object.defineProperty(target, prop, Object.getOwnPropertyDescriptor(source, prop));
        });
    };
    mixins.forEach((mixin) => {
        // outside contructor() to allow aggregation(A,B,C).staticFunction() to be called etc.
        copyProps(base.prototype, mixin.prototype);
        copyProps(base, mixin);
    });
    return base;
};
exports.mixins = mixins;
